#!/usr/bin/env python
# -*- coding: utf-8 -*-

import rospy, cv2, cv_bridge, numpy
from sensor_msgs.msg import Image
from geometry_msgs.msg import Twist

from std_srvs.srv import Empty
from naoqi_bridge_msgs.msg import JointAnglesWithSpeed


class Follower:
  def __init__(self):
    self.bridge = cv_bridge.CvBridge()
    cv2.namedWindow("window", 1)

    self.cmd_vel_pub = rospy.Publisher('cmd_vel', Twist, queue_size = 1)
    self.twist = Twist()

    # 首は少し下に
    self.joint_angles_pub = rospy.Publisher('joint_angles', JointAnglesWithSpeed, queue_size = 1)
    self.headAngles = JointAnglesWithSpeed()
    self.headAngles.header.frame_id = 'Head'
    self.headAngles.joint_names = ['HeadYaw', 'HeadPitch']
    self.headAngles.joint_angles = [0.0, 0.25]
    self.headAngles.speed = 0.2
    self.headAngles.relative = 0
    self.joint_angles_pub.publish(self.headAngles)

    # カメラ画像取得開始
    self.image_sub = rospy.Subscriber('pepper_robot/camera/bottom/image_raw', 
                                      Image, self.image_callback)


  def image_callback(self, msg):
    # カメラ画像の取得、黄色でフィルター
    image = self.bridge.imgmsg_to_cv2(msg,desired_encoding='bgr8')

    hsv = cv2.cvtColor(image, cv2.COLOR_BGR2HSV)
    lower_yellow = numpy.array([ 20,  50,  50])
    upper_yellow = numpy.array([40, 255, 255])
    mask = cv2.inRange(hsv, lower_yellow, upper_yellow)

    masked = cv2.bitwise_and(image, image, mask=mask)

    #　画像の高さ3/4の位置から20行分だけを抜き出す 
    h, w, d = image.shape
    search_top = 3*h/4
    search_bot = search_top + 20
    mask[0:search_top, 0:w] = 0
    mask[search_bot:h, 0:w] = 0

    # マスクされた画像の中心点を探す
    M = cv2.moments(mask)
    if M['m00'] > 0:

      # 中心点が見つかった時、その中心点に向かって移動
      cx = int(M['m10']/M['m00'])
      cy = int(M['m01']/M['m00'])
      cv2.circle(image, (cx, cy), 20, (0,0,255), -1)
      err = cx - w/2
      self.twist.linear.x =  0.2 
      self.twist.angular.z = -float(err) / 100.0
      self.cmd_vel_pub.publish(self.twist)
      self.joint_angles_pub.publish(self.headAngles)

    cv2.imshow("window", image)
    cv2.waitKey(1)


# 自律モードを停止する
disable_alife = rospy.ServiceProxy('pepper_robot/pose/life/disable', Empty)
#disable_alife()

# モーターを ON
wakeup = rospy.ServiceProxy('pepper_robot/pose/wakeup', Empty)
wakeup()

#　黄色い線に沿って走行開始
print 'Start following yellow line..'
rospy.init_node('follower')

follower = Follower()

rospy.spin()

